import time

from flask import Flask, request
from gevent import monkey
from playwright.sync_api import sync_playwright
from dotenv import load_dotenv
import os
# 加载环境变量
load_dotenv()
monkey.patch_all()

app = Flask(__name__)

global_a1 = ""
# 确保在模块级别声明全局变量
browser_context = None
context_page = None


def get_context_page(instance, stealth_js_path):
    chromium = instance.chromium
    browser = chromium.launch(headless=True)
    context = browser.new_context()
    context.add_init_script(path=stealth_js_path)
    page = context.new_page()
    return context, page


# 如下更改为 stealth.min.js 文件路径地址
stealth_js_path = "stealth.min.js"
print("正在启动 playwright")
playwright = sync_playwright().start()
browser_context, context_page = get_context_page(playwright, stealth_js_path)
context_page.goto("https://www.xiaohongshu.com")
print("正在跳转至小红书首页")
time.sleep(5)
context_page.reload()
time.sleep(1)
cookies = browser_context.cookies()
for cookie in cookies:
    if cookie["name"] == "a1":
        global_a1 = cookie["value"]
        print("当前浏览器 cookie 中 a1 值为：" + cookie["value"] + "，请将需要使用的 a1 设置成一样方可签名成功")
print("跳转小红书首页成功，等待调用")


def sign(uri, data, a1, web_session):
    global browser_context, context_page  # 声明全局变量
    global global_a1
    try:
        # 确保页面仍然有效，如果页面已关闭则重新初始化
        if context_page is None or context_page.is_closed():
            browser_context, context_page = get_context_page(playwright, stealth_js_path)
            context_page.goto("https://www.xiaohongshu.com")
            time.sleep(5)
            context_page.reload()
            time.sleep(1)
            
        if a1 != global_a1:
          browser_context.add_cookies([
              {'name': 'a1', 'value': a1, 'domain': ".xiaohongshu.com", 'path': "/"}
          ])
          context_page.reload()
          time.sleep(1)
          global_a1 = a1
          # 执行 JavaScript 函数
        encrypt_params = context_page.evaluate("([url, data]) => window._webmsxyw(url, data)", [uri, data])
        return {
            "x-s": encrypt_params["X-s"],
            "x-t": str(encrypt_params["X-t"])
            "encrypt_params": encrypt_params,
        }
    except Exception as e:
        print(f"Error during sign operation: {e}")
        return {"error": str(e)}


@app.route("/sign", methods=["POST"])
def hello_world():
    json = request.json
    uri = json["uri"]
    data = json["data"]
    a1 = json["a1"]
    web_session = json["web_session"]
    return sign(uri, data, a1, web_session)


@app.route("/a1", methods=["GET"])
def get_a1():
    global global_a1
    return {'a1': global_a1}


if __name__ == '__main__':
    port = os.getenv('XHS_API_PORT', 5005)
    app.run(host="0.0.0.0", port=port)